@@ -3,4 +3,4 @@ THIS SPECIFIC FILE IS DISTRIBUTED UNDER THE UNLICENSE: http://unlicense.org.
 
 """
 
-from .main.parser import main
+# from .main.parser import main
